;
;   Module name      :	LogObj.au3
;   Version          :	1.1
;   Started on       :	February 14, 2013
;   Completed on     :	February 17, 2013
;   Author           :  J.Hubb
;
;   Module type      :	Abstract object
;   Description      :	Log objects handler
;
;   Developed under  :	AutoIt 3.2.10.0
;   Sw requirements  :	None
;
;   Parameters       :	None
;   Notes - usage    :	Log objects can write logs, in the Syslog format, to file or server; each object
;						can be set to one of 3 different modes:
;
;						$LO_FILE_MODE: writes log messages to the file, specified during LOInit() call, and
;							prefixes to the message a timestamp, whose format can be changed, the hostname,
;							the script name, the script Pid, a priority (if different from the default one
;							chosen in LOInit) and an optional user tag:
;								"TimeStamp HostName ApplicationName[ProcessId]: [Priority] OptionalTag: Msg"
;							With std config. the format is similar to the one used by a Syslog server.
;
;						$LO_SYSLOG_MODE: sends log messages to the Syslog server, specified during LOInit()
;							call, using "RFC5424 Syslog protocol" for Udp and "Cisco PIX syslog over TCP
;							protocol" for Tcp; messages can be prefixed by an optional user tag; Syslog
;							server will usually store the log message in the format:
;								"Timestamp HostName ApplicationName[ProcessId]: OptionalTag: Msg"
;							The priority does not appear since it is used by Syslog server to store log
;							messages to different files or databases.
;
;						$LO_INI_MODE: the library can read the .ini file, specified during LOInit() call,
;							to retrieve the effective log mode ($LO_FILE_MODE or $LO_SYSLOG_MODE) and its
;							associated data; once data are read, the behavior is the same as described above.
;							The advantage of using a .ini file is that the destination of log messages (of
;							a script) can be changed without modifying the source but just the .ini file;
;							the same .ini file can be used for many scripts, each of them using a different
;							section (or even sharing the same one).
;							Every section of .ini file, can contain the following variables:
;								Mode	Log mode; it can be $LO_FILE_MODE, "file", $LO_SYSLOG_MODE, "server"
;								Dest	Log destination; it can be: for a log file, a file path; for a log
;										server, a string such as: [@[@]]TcpNameOrIp[:PortNum]
;								Pri		Log default priority; according to RFC5424, it can be a number or a
;										string in the form <Facility>.<Severity>
;								DTFmt	Date/Time format for file mode; it can be a string as $LO_..._DT
;										constants or a function name preceeded by the string "Func:"
;
;						Priorities are defined, as RFC5424 describes, by the couple facility.severity, for
;						Syslog mode and for file mode too.
;
;						It is possible to use many log objects at the same time, since each one is defined
;						by a handle built by LOInit and destroyed by LODone; it is a good practice to call
;						LODone always at the end of the script for each log object used.
;
;						The library exports 5 functions (for details and remarks on parameter values, see
;						comments at the beginning of each function):
;							LOInit			initializes a log object, defining mode and parameters
;							LOWrite			writes a log message to the chosen (in LOInit) destination
;							LOWriteArray	writes an array of log messages to the chosen destination
;							LODone			destroys the log object and closes communications opened
;							LOSyslogDT		simple date/time function to have a timestamp like Syslog one
;
;						Warning: the AutoIt option "ExpandVarStrings" must be set to 0 (that is the
;						default setting) before calling any library functions, otherwise date/time values
;						could be easily mistaken.
;
;						Notice: date/time values are always sent in UTC to Syslog server: the result is
;						correct even if residing not in UTC countries; anyway it is possible to modify
;						this settings specifying a simple date/time user function (see LOSyslogDT as an
;						example) when calling LOInit.
;


;--- Global directives -----------------------------------------------------------------------------



#include-once

Opt("MustDeclareVars", 1)



;--- Include files ---------------------------------------------------------------------------------



;--- Public constants ------------------------------------------------------------------------------



const $LO_FILE_NAME = "LogObj.au3"
const $LO_MODULE_VER = "1.1"
const $LO_MODULE_NAME = $LO_FILE_NAME & " " & $LO_MODULE_VER & "; Open Source library on February 15, 2013 by J.Hubb."

; Error codes
const $LO_NO_ERROR	= 0				; No error
const $LO_INI_ERR	= 1				; File .ini not found or needed section not existant or not complete
const $LO_MODE_ERR	= 2				; Log mode unknown
const $LO_FILE_ERR	= 3				; Errors during writing to log file
const $LO_PRI_ERR	= 4				; Priority value is invalid
const $LO_FUNC_ERR	= 5				; Date/time function does not exist or does not return a value
const $LO_PROT_ERR	= 6				; Protocol not specified for server in Syslog mode
const $LO_DRV_ERR	= 7				; Udp or Tcp driver returned an error
const $LO_SRV_ERR	= 8				; Syslog server returned an error during connection or writing
const $LO_HNDL_ERR	= 9				; Log object handle is invalid
const $LO_ARRAY_ERR	= 10			; Message array is not an array

; Log mode
const $LO_FILE_MODE		= "0"
const $LO_SYSLOG_MODE	= "1"
const $LO_INI_MODE		= "2"

; Default .ini name
const $LO_DEF_INI = "LogObj.ini"

; Key names in .ini (see parameters of LOInit for details)
const $LO_KEY_MODE	= "Mode"		; Log mode
const $LO_KEY_DEST	= "Dest"		; Log destination (file or server)
const $LO_KEY_PRI	= "Pri"			; Log default priority
const $LO_KEY_DTFMT	= "DTFmt"		; Date/Time format for file mode

; Date/time common formats
const $LO_SYSLOG_DT	= "@Year@-@Mon@-@MDay@T@Hour@:@Min@:@Sec@Z"		; Syslog requested timestamp format
const $LO_YMD_DT	= "@Year@-@Mon@-@MDay@ @Hour@:@Min@:@Sec@"
const $LO_DMY_DT	= "@MDay@-@Mon@-@Year@ @Hour@:@Min@:@Sec@"
const $LO_MDY_DT	= "@Mon@-@MDay@-@Year@ @Hour@:@Min@:@Sec@"

; Prefix to specify a date/time function (see parameter $dt_fmt of LOInit for details)
const $LO_FUNC_DT	= "Func:"

; Facility symbolic values (see RFC5424)
const $LO_FAC_NUM = 24
const $LO_FAC[$LO_FAC_NUM] = [ _
		"kern", "user", "mail", "daemon", "auth", "syslog", "lpr", "news", "uucp", "cron", _
		"authpriv", "ftp", "ntp", "logaudit", "logalert", "cron2", "local0", "local1", "local2", "local3", _
		"local4", "local5", "local6", "local7" _
		]

; Severity symbolic values (see RFC5424)
const $LO_SEV_NUM = 8
const $LO_SEV[$LO_SEV_NUM] = ["emerg", "alert", "crit", "err", "warning", "notice", "info", "debug"]

; Default and max priority
const $LO_DEF_PRI = "user.notice"
const $LO_MAX_PRI = ($LO_FAC_NUM - 1) * 8 + $LO_SEV_NUM - 1

; Default port number on Udp and Tcp Syslog protocol
const $LO_DEF_PORT = 514



;--- Public variables ------------------------------------------------------------------------------



; Short names for monthes, used in LOSyslogDT(); user can re-assign this variable to change them
; or can create and use another function
global $LO_mmm[12] = ["Jan", "Feb", "Mar", "Apr", "Maj", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]



;--- Private constants -----------------------------------------------------------------------------



; Indexes for fields in a Handle array (mode, numeric_priority, symbolic_priority, mode_data);
; last element is only used to calculate the number of elements
enum $_LO_H_MODE, $_LO_H_NPRI, $_LO_H_SPRI, $_LO_H_DATA, $_LO_HANDLE_DIM

; Indexes for fields in a Data array in $LO_FILE_MODE (destination, date_time_format,
; date_time_function); last element is only used to calculate the number of elements
enum $_LO_FD_DEST, $_LO_FD_DT, $_LO_FD_FUNC, $_LO_FILE_DATA_DIM

; Indexes for fields in a Data array in $LO_SYSLOG_MODE (protocol, socket); last element is only
; used to calculate the number of elements
enum $_LO_SD_PROT, $_LO_SD_SOCK, $_LO_SYSLOG_DATA_DIM



;--- Private variables -----------------------------------------------------------------------------



; UDP or TCP ever used?
global $_LO_udp_used = False
global $_LO_tcp_used = False

; Previous OnExit handler
global $_LO_prev_onexit = ""



;--- Public functions code -------------------------------------------------------------------------



func LODone(byref $h)
;---
; Dispose a log object, deleting its handle and closing eventual network sockets.
;
; $h	log object handle
; ret	1 on success, 0 on failure setting @error
;---
	local $data
	
	if (not IsArray($h)) or (UBound($h) <> $_LO_HANDLE_DIM) then
		SetError($LO_HNDL_ERR)
		return 0
	endif
	switch $h[$_LO_H_MODE]
		case $LO_FILE_MODE
		case $LO_SYSLOG_MODE
			$data = $h[$_LO_H_DATA]
			if ($data[$_LO_SD_PROT] == "u") then		; Udp protocol
				if (UDPCloseSocket($data[$_LO_SD_SOCK]) <> 1) then
					SetError($LO_SRV_ERR)
					return 0
				endif
			else										; Tcp protocol
				if (TCPCloseSocket($data[$_LO_SD_SOCK]) <> 1) then
					SetError($LO_SRV_ERR)
					return 0
				endif
			endif
	endswitch
	$h = 0
	return 1
endfunc



func LOInit($mode, $dest, $pri = Default, $dt_fmt = Default)
;---
; Initialize a log object, returning its handle; on error sets @error
;
; $mode		mode of log storage; it can be:
;				$LO_FILE_MODE	the log is a file
;				$LO_SYSLOG_MODE	the log is send to a Syslog server
;				$LO_INI_MODE	log mode and dest are read from a .ini file
; $dest		destination of log; according to $mode value, it must be:
;				$LO_FILE_MODE	complete pathname of log file; environment variables are allowed
;				$LO_SYSLOG_MODE	destination server (IP or Tcp name); it must be prefixed with "@" for
;								Udp protocol or "@@" for Tcp protocol; it can be followed by the port
;								number in the form ":PortNum" (default is $LO_DEF_PORT)
;				$LO_INI_MODE	complete pathname of .ini file; environment variables are allowed;
;								if "" is specified, then $LO_DEF_INI is searched first in script dir.
;								and, if not found, in <include> dir.
; $pri		default priority of messages, expressed numerically or as a string "facility.severity"; the
;			default value is $LO_DEF_PRI; according to $mode value, the behavior is:
;				$LO_FILE_MODE	the specified default priority is never written by LOWrite() to the log file
;				$LO_SYSLOG_MODE	the default priority is used in sending messages by LOWrite when not
;								specified otherwise
;				$LO_INI_MODE	this value is used to specify the section of .ini to be read; the real
;								priority is read from .ini
; $dt_fmt	this value is only used in $LO_FILE_MODE: date and time format sent to the log; the default
;			value is $LO_YMD_DT; valid values are strings as $LO_..._DT constants or the name of a callback
;			function with no parameter as LOSyslogDT(), specified as "Func:LOSyslogDT" (see $LO_FUNC_DT)
; ret		the handle of the log object
;---
	local $prev_opt, $res, $hndl[$_LO_HANDLE_DIM]

	if ($mode == $LO_INI_MODE) then
		if ($dest = "") then
			$dest = _LOSearchStdDir($LO_DEF_INI)
		else
			$prev_opt = AutoItSetOption("ExpandEnvStrings", 1)
			$dest = $dest
			AutoItSetOption("ExpandEnvStrings", $prev_opt)
		endif
		if (not FileExists($dest)) or (not _LOReadIni($dest, $pri, $mode, $dest, $pri, $dt_fmt)) then
			SetError($LO_INI_ERR)
			return 0
		endif
	endif
	if ($pri == Default) then
		$pri = $LO_DEF_PRI
	endif
	if ($dt_fmt == Default) then
		$dt_fmt = $LO_YMD_DT
	endif
	switch $mode
		case $LO_FILE_MODE
			$prev_opt = AutoItSetOption("ExpandEnvStrings", 1)
			$dest = $dest
			AutoItSetOption("ExpandEnvStrings", $prev_opt)
			$res = _LOInitFile($dest, $pri, $dt_fmt, $hndl)
		case $LO_SYSLOG_MODE
			$res = _LOInitSyslog($dest, $pri, $hndl)
		case else
			$res = $LO_MODE_ERR
	endswitch
	if ($res <> $LO_NO_ERROR) then
		SetError($res)
		return 0
	endif
	return $hndl
endfunc



func LOSyslogDT()
;---
; Returns the current Date/Time value formatted as Syslog daemon does. This function can be
; passed to LOInit() function in the $dt_fmt parameter.
;
; ret	current date/time formatted as Syslogd format "MMM DD hh:mm:ss"
;---
	return $LO_mmm[@MON - 1] & " " & @MDAY & " " & @HOUR & ":" & @MIN & ":" & @SEC
endfunc



func LOWrite($h, $msg, $tag = Default, $pri = Default)
;---
; Writes a message and an optional tag to the log object, using the optional priority; a line in the
; log has the form: "TimeStamp HostName ApplicationName[ProcessId]: OptionalTag: Message".
;
; $h	log object handle
; $msg	the message to be written to the log
; $tag	an optional marker that is inserted before the message
; $pri	an optional priority, expressed numerically or as a string "facility.severity", to be used
;		when writing the message to the log object; if the priority is an invalid value, the default
;		one is used and @error is set; exact behavior depends on log mode:
;			$LO_FILE_MODE	if the specified priority is the same of default one set in LOInit,
;							then it is ignored, otherwise it is written before the message and before
;							the optional tag with the form "[pri="facility.severity"] "
;			$LO_SYSLOG_MODE	the specified priority is always used when sending the message to the server
; ret	1 on success, 0 on failure setting @error
;---
	local $num_p, $sym_p, $err = $LO_NO_ERROR, $data, $prev_opt, $dt
	
	if (not IsArray($h)) or (UBound($h) <> $_LO_HANDLE_DIM) then
		SetError($LO_HNDL_ERR)
		return 0
	endif
	if ($tag <> Default) then
		$msg = $tag & ": " & $msg
	endif
	if ($pri == Default) then
		$num_p = $h[$_LO_H_NPRI]
	else
		if (not _LOChkPriority($pri, $num_p, $sym_p)) then
			$num_p = $h[$_LO_H_NPRI]				; Ignore wrong priority and write with default one
			$err = $LO_PRI_ERR
		endif
	endif
	$data = $h[$_LO_H_DATA]
	switch $h[$_LO_H_MODE]
		case $LO_FILE_MODE
			if ($data[$_LO_FD_DT] <> "") then		; Use a date/time string
				$prev_opt = AutoItSetOption("ExpandVarStrings", 1)
				$dt = $data[$_LO_FD_DT]
				AutoItSetOption("ExpandVarStrings", $prev_opt)
			else									; Use a date/time function
				$dt = Call($data[$_LO_FD_FUNC])
			endif
			if ($num_p == $h[$_LO_H_NPRI]) then
				$msg = $dt & " " & @ComputerName & " " & @ScriptName & "[" & @AutoItPID & "]: " & $msg
			else
				$msg = $dt & " " & @ComputerName & " " & @ScriptName & "[" & @AutoItPID & "]: " & _
						"[pri=""" & $sym_p & """] " & $msg
			endif
			if (FileWriteLine($data[$_LO_FD_DEST], $msg) == 0) then
				SetError($LO_FILE_ERR)
				return 0
			endif
		case $LO_SYSLOG_MODE
			$prev_opt = AutoItSetOption("ExpandVarStrings", 1)
			$dt = $LO_SYSLOG_DT
			AutoItSetOption("ExpandVarStrings", $prev_opt)
			$msg = "<" & $num_p & ">1 " & $dt & " " & @ComputerName & " " & @ScriptName & " " & _
					@AutoItPID & " - : " & $msg
			if ($data[$_LO_SD_PROT] == "u") then	; Udp protocol
				if (UDPSend($data[$_LO_SD_SOCK], $msg) == 0) then
					SetError($LO_SRV_ERR)
					return 0
				endif
			else									; Tcp protocol
				if (TCPSend($data[$_LO_SD_SOCK], $msg & @LF) == 0) then
					SetError($LO_SRV_ERR)
					return 0
				endif
			endif
	endswitch
	SetError($err)
	return 1
endfunc



func LOWriteArray($h, $a, $pre = Default, $post = Default, $tag = Default, $pri = Default)
;---
; Writes all the messages contained in the array $a, each one prefixed with $pre and followed by $post; see
; LOWrite() for the meaning of left parameters.
;
; $h	log object handle
; $a	array of messages to be written to the log
; $pre	a string prefix to each message; default is the empty string
; $post	a string that follows each message; default is the empty string
; ...	for other parameters, see LOWrite()
; ret	1 on success, 0 on failure setting @error
;---
	local $err
	
	if (not IsArray($a)) then
		SetError($LO_ARRAY_ERR)
		return 0
	endif
	for $m in $a
		if ($pre <> Default) then
			$m = $pre & $m
		endif
		if ($post <> Default) then
			$m = $m & $post
		endif
		LOWrite($h, $m, $tag, $pri)
		$err = @error
		if ($err <> $LO_NO_ERROR) then
			SetError($err)
			return 0
		endif
	next
	SetError($LO_NO_ERROR)
	return 1
endfunc



;--- Private functions code ------------------------------------------------------------------------



func _LOAddTrailingBS($d)
;---
; Adds a trailing backslash if $d has not already (i.e. on root dir.).
;
; $d	directory where add tralinig backslash
; ret	directory $d with a trailing backslash
;---
	if (StringRight($d, 1) == "\") then
		return $d
	else
		return $d & "\"
	endif
endfunc



func _LOBasename($p)
;---
; Returns the basename (filename) of the specified path.
;
; $p	complete or partial pathname
; ret	basename of $p
;---
	return StringTrimLeft($p, StringInStr($p, "\", 0, -1))
endfunc



func _LOChkPriority($p, byref $num_p, byref $sym_p)
;---
; If $p is numeric, then it is checked to be within allowed range; if $p is a string, then it must be in
; the form "facility.severity"; Facility and Severity must be among standard values (see $LO_FAC and $LO_SEV).
; If $p is an allowed value, its numeric and symbolic values are returned.
;
; $p		priority value or string in the form "facility.severity"
; $num_p	returned to caller: numeric priority value, according to RFC5424
; $sym_p	returned to caller: symbolic priority value in form "facility.severity"
; ret		1 on success, 0 on errors
;---
	local $part, $f = 0, $s = 0

	if (String(Number($p)) == $p) then
		$p = Number($p)
		if (($p >= 0) and ($p <= $LO_MAX_PRI)) then
			$num_p = $p
			$sym_p = $LO_FAC[BitShift($p, 3)] & "." & $LO_SEV[BitAnd($p, 7)]
			return 1
		else
			return 0
		endif
	endif
	if (isString($p)) then
		$part = StringSplit($p, ".")
		if ($part[0] <> 2) then
			return 0
		endif
		do
			if ($part[1] = $LO_FAC[$f]) then		; "=" is case insensitive
				ExitLoop
			endif
			$f += 1
		until $f == $LO_FAC_NUM
		if ($f == $LO_FAC_NUM) then
			return 0
		endif
		do
			if ($part[2] = $LO_SEV[$s]) then		; "=" is case insensitive
				ExitLoop
			endif
			$s += 1
		until $s == $LO_SEV_NUM
		if ($s == $LO_SEV_NUM) then
			return 0
		endif
		$num_p = $f * 8 + $s
		$sym_p = $LO_FAC[BitShift($num_p, 3)] & "." & $LO_SEV[BitAnd($num_p, 7)]
		return 1
	endif
	return 0
endfunc



func _LOInitFile($d, $p, $dt, byref $h)
;---
; Initialize a file log object, checking it and returning its handle in $h.
;
; $d	pathname of log file
; $p	default priority of messages, expressed as the string "facility.severity"
; $dt	date/time format sent to the log; valid values are strings as $LO_..._DT constants or the name of
;		a callback function with no parameter as LOSyslogDT(), specified as "Func:LOSyslogDT" (see $LO_FUNC_DT)
; $h	returned to caller: handle of the file log object
; ret	error code on errors or $LO_NO_ERROR on success
;---
	local $t, $data[$_LO_FILE_DATA_DIM], $len = StringLen($LO_FUNC_DT)
	
	$h[$_LO_H_MODE] = $LO_FILE_MODE
	if (not FileWrite($d, "")) then
		return $LO_FILE_ERR
	endif
	$data[$_LO_FD_DEST] = _LORealPath($d)
	if (not _LOChkPriority($p, $h[$_LO_H_NPRI], $h[$_LO_H_SPRI])) then
		return $LO_PRI_ERR
	endif
	if (StringLeft($dt, $len) = $LO_FUNC_DT) then	; "=" is case insensitive
		$dt = StringTrimLeft($dt, $len)
		$t = Call($dt)
		if (@error) or ($t == "") then
			return $LO_FUNC_ERR
		endif
		$data[$_LO_FD_FUNC] = $dt
	else
		$data[$_LO_FD_DT] = $dt
	endif
	$h[$_LO_H_DATA] = $data
	return $LO_NO_ERROR
endfunc



func _LOInitSyslog($d, $p, byref $h)
;---
; Initialize a Syslog log object, checking it and returning its handle in $h.
;
; $d	destination server (IP or Tcp name); it must be prefixed with "@" for Udp protocol or "@@" for
;		Tcp protocol; it can be followed by the port number in the form ":PortNum" (default is $LO_DEF_PORT)
; $p	default priority of messages, expressed as the string "facility.severity"
; $h	returned to caller: handle of the Syslog log object
; ret	error code on errors or $LO_NO_ERROR on success
;---
	local $data[$_LO_SYSLOG_DATA_DIM], $t, $port = $LO_DEF_PORT, $socket
	
	$h[$_LO_H_MODE] = $LO_SYSLOG_MODE
	if (StringLeft($d, 1) <> "@") then
		return $LO_PROT_ERR
	endif
	if (StringLeft($d, 2) == "@@") then
		$data[$_LO_SD_PROT] = "t"
		$d = StringTrimLeft($d, 2)
	else
		$data[$_LO_SD_PROT] = "u"
		$d = StringTrimLeft($d, 1)
	endif
	$t = StringSplit($d, ":")
	if ($t[0] > 1) then
		$d = $t[1]
		$port = $t[2]								; Ignore eventual following ":"
	endif
	if (not $_LO_tcp_used) then						; Start always Tcp to resolve Tcp names
		if (TCPStartup()) then
			$_LO_tcp_used = True
		else
			return $LO_DRV_ERR
		endif
	endif
	if ($data[$_LO_SD_PROT] == "u") then			; Udp protocol
		if (not $_LO_udp_used) then
			if (UDPStartup()) then
				$_LO_udp_used = True
			else
				return $LO_DRV_ERR
			endif
		endif
		$d = TCPNameToIp($d)						; If $d is already an Ip, $d is returned
		$socket = UDPOpen($d, $port)
	else											; Tcp protocol
		$d = TCPNameToIp($d)						; If $d is already an Ip, $d is returned
		$socket = TCPConnect($d, $port)
	endif
	if ($socket == -1) then
		return $LO_SRV_ERR
	endif
	$data[$_LO_SD_SOCK] = $socket
	if (not _LOChkPriority($p, $h[$_LO_H_NPRI], $h[$_LO_H_SPRI])) then
		return $LO_PRI_ERR
	endif
	$h[$_LO_H_DATA] = $data
	return $LO_NO_ERROR
endfunc



func _LOOnExit()
;---
; Module OnExit handler; stops started network services.
;
; ret	nothing
;---
	if ($_LO_udp_used) then
		UDPShutdown()
	endif
	if ($_LO_tcp_used) then
		TCPShutdown()
	endif
	Call($_LO_prev_onexit)
endfunc



func _LOReadIni($ini, $sect, byref $mode, byref $dest, byref $pri, byref $dt_fmt)
;---
; Reads, from specified .ini, the values from the section $sect; no syntax checkin
; is performed on read values.
;
; $ini		complete filepath of .ini file
; $sect		name of section in .ini
; $mode		returned to caller: mode read from .ini
; $dest		returned to caller: destination read from .ini
; $pri		returned to caller: priority read from .ini or Default
; $dt_fmt	returned to caller: date/time format read from .ini or Default
; ret		1 on success, 0 on errors
;---
	switch StringLower(IniRead($ini, $sect, $LO_KEY_MODE, ""))
		case $LO_FILE_MODE, "file"
			$mode = $LO_FILE_MODE
		case $LO_SYSLOG_MODE, "syslog"
			$mode = $LO_SYSLOG_MODE
		case else
			return 0
	endswitch
	$dest = IniRead($ini, $sect, $LO_KEY_DEST, "")
	if ($dest == "") then
		return 0
	endif
	$pri = IniRead($ini, $sect, $LO_KEY_PRI, "")
	if ($pri == "") then
		$pri = Default
	endif
	$dt_fmt = IniRead($ini, $sect, $LO_KEY_DTFMT, "")
	if ($dt_fmt == "") then
		$dt_fmt = Default
	endif
	return 1
endfunc



func _LORealPath($p)
;---
; Returns the real path for relative (or absolute) path $p; path $p must be
; a valid existent path or a wrong result will be returned.
;
; $p	complete or partial pathname or dirname
; ret	real path for $p
;---
	local $w = @WorkingDir
	
	FileChangeDir($p & "\..")
	$p = _LOAddTrailingBS(@WorkingDir) & _LOBaseName($p)
	FileChangeDir($w)
	return $p
endfunc



func _LOSearchStdDir($n)
;---
; Searches file $n in script directory and, if not found, in <include> directory. Returns
; complete $n path or "" if not found.
;
; ret	complete $n pathname or "" if not found
;---
	local $p
	
	$p = _LOAddTrailingBS(@ScriptDir) & $n			; Handle root dir. (needed on W9x)
	if (not FileExists($p)) then
		$p = @AutoItExe & "\..\include\" & $n
		if (not FileExists($p)) then
			return ""
		endif
	endif
	return $p
endfunc



;--- Initialization code ---------------------------------------------------------------------------



$_LO_prev_onexit = Opt("OnExitFunc", "_LOOnExit")	; Set the module OnExit handler
